//========================================================================================
//
//	File:				Commands.h
//	Release Version:	$ ODF 1 $
//
//	Copyright:			(c) 1993 - 1996 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef COMMANDS_H
#define COMMANDS_H

// ----- Framework Includes -----

#ifndef FWCLPCMD_H
#include "FWClpCmd.h"
#endif

#ifndef FWDRCMD_H
#include "FWDrCmd.h"
#endif

#ifndef FWBNDSTR_H
#include "FWBndStr.h"
#endif

//========================================================================================
// Forward Declarations
//========================================================================================

#include "Content.h"

//========================================================================================
// CODFxEditCommand
//========================================================================================

class CODFxEditCommand : public FW_CClipboardCommand
{
//----------------------------------------------------------------------------------------
//	Initialization/Destruction
//
public:

	FW_DECLARE_AUTO(CODFxEditCommand)
	
	CODFxEditCommand(Environment* ev,
					  ODCommandID id,
					  CODFxContent* itsContent,
					  FW_CFrame* frame);

	virtual ~CODFxEditCommand();

//----------------------------------------------------------------------------------------
//	Inherited API
//
public:
	virtual void		UndoIt(Environment *ev);
	virtual void		RedoIt(Environment *ev);
	virtual void		SaveUndoState(Environment *ev);

//----------------------------------------------------------------------------------------
//	New API
//
private:
	void 				RemoveSelection(Environment* ev);
	void 				RestoreSelection(Environment* ev);
	void 				SwapSelection(Environment* ev);

//----------------------------------------------------------------------------------------
//	Data Members
//
private:
	CODFxContent*					fODFxContent;
	CODFxContent::ODFXContentStruct	fSavedData;			// Saved data for undo/redo
};



//========================================================================================
//	class CODFxDragCommand
//========================================================================================

class CODFxDragCommand : public FW_CDragCommand
{
//----------------------------------------------------------------------------------------
//	Initialization/Destruction
//
public:

	FW_DECLARE_AUTO(CODFxDragCommand)
	
	CODFxDragCommand(Environment* ev,
					  CODFxContent* content,
					  FW_CFrame* frame);
	virtual ~CODFxDragCommand();

//----------------------------------------------------------------------------------------
//	Inherited API
//
public:
	virtual void		UndoIt(Environment *ev);
	virtual void		RedoIt(Environment *ev);
	virtual void		SaveUndoState(Environment *ev);

//----------------------------------------------------------------------------------------
//	Data Members
//
private:
	CODFxContent*					fODFxContent;
	CODFxContent::ODFXContentStruct	fSavedData;			// Saved data for undo/redo
};

//========================================================================================
//	class CODFxDropCommand
//========================================================================================

class CODFxDropCommand : public FW_CDropCommand
{
//----------------------------------------------------------------------------------------
//	Initialization/Destruction
//
public:

	FW_DECLARE_AUTO(CODFxDropCommand)
	
	CODFxDropCommand(Environment *ev,
					  CODFxContent* content,
					  FW_CFrame* frame,
					  ODDragItemIterator* dropInfo, 
					  ODFacet* odFacet,
					  const FW_CPoint& dropPoint);

	virtual ~CODFxDropCommand();

//----------------------------------------------------------------------------------------
//	Inherited API
//
public:
	virtual void		UndoIt(Environment *ev);
	virtual void		RedoIt(Environment *ev);
	virtual void		SaveUndoState(Environment *ev);

//----------------------------------------------------------------------------------------
//	New API
//
private:
	void SwapSelection(Environment* ev);

//----------------------------------------------------------------------------------------
//	Data Members
//
private:
	CODFxContent*					fODFxContent;
	CODFxContent::ODFXContentStruct	fSavedData;			// Saved data for undo/redo
};


#endif
